Imports System.Configuration
Imports System.Diagnostics.Contracts
Imports System.Web.Mvc
Imports System.Web.Security
Imports SocialCoding.FbAuth.Framework

Namespace SocialCoding.FbAuth.Controllers
	Public Class AuthController
		Inherits Controller
		''' <summary>
		''' Starts the process that authenticates users via Facebook
		''' </summary>
		''' <returns>Redirect to FB</returns>
		Public Function FacebookLogin() As ActionResult
			Contract.Requires(Of ArgumentNullException)(Request.Url IsNot Nothing)

			Dim returnUri = New UriBuilder(Request.Url) With {.Path = Url.Action("FacebookAuthenticated", "Auth")}

			Dim client = New FacebookClient
			Dim fbLoginUri = client.GetLoginUrl(New With {Key .client_id = ConfigurationManager.AppSettings("fb_key"), Key .redirect_uri = returnUri.Uri.AbsoluteUri, Key .response_type = "code", Key .display = "popup", Key .scope = "email,publish_stream"})


			Return Redirect(fbLoginUri.ToString)
		End Function

		''' <summary>
		''' Handles the post-authentication step from Facebook
		''' </summary>
		''' <param name="returnUrl">Return URL, if any</param>
		''' <returns>Redirect to home/return URL</returns>
		Public Function FacebookAuthenticated(ByVal returnUrl As String) As ActionResult
			Contract.Requires(Of ArgumentNullException)(Request.Url IsNot Nothing)

			Dim redirectUri = New UriBuilder(Request.Url) With {.Path = Url.Action("FacebookAuthenticated", "Auth")}
			Dim client = New FacebookClient
			Dim oauthResult = client.ParseOAuthCallbackUrl(Request.Url)

			' Exchange the code for an access token    
            Dim result As Object = client.Get("/oauth/access_token", New With {Key .client_id = ConfigurationManager.AppSettings("fb_key"), Key .client_secret = ConfigurationManager.AppSettings("fb_secret"), Key .redirect_uri = redirectUri.Uri.AbsoluteUri, Key .code = oauthResult.Code})

            Dim token = CType(result, Object).access_toke

            FbHelpers.AccessTokenSave(Response, CStr(token))

			Dim user As Object = client.Get("/me", New With {Key .fields = "first_name,last_name,email", Key .access_token = token})

            'FormsAuthentication.SetAuthCookie(user.email, false)
            Dim userName= String.Format("{0} {1}", user.first_name, user.last_name)
            Dim cookie = FormsAuthentication.GetAuthCookie(CStr(userName), True)
			Response.AppendCookie(cookie)

			Return Redirect(If(returnUrl, "/"))
		End Function

		''' <summary>
		''' Logs off and redirects to the specified view.
		''' </summary>
		''' <param name="defaultAction">Action to redirect</param>
		''' <param name="defaultController">Controller to redirect</param>
		''' <returns>Redirect</returns>
		Public Function LogOff(Optional ByVal defaultAction As String = "Index", Optional ByVal defaultController As String = "Home") As ActionResult
			FormsAuthentication.SignOut()
			FbHelpers.AccessTokenDelete(Request)
			Return RedirectToAction(defaultAction, defaultController)
		End Function
	End Class
End Namespace
